# coding=utf-8
import re
from collections import defaultdict


def match(abnex_str, to_match):
	regex = parse(abnex_str)
	return re.match(regex, to_match)


def holds(abnex_str, to_match):
	return bool(match(abnex_str, to_match))


def contains(abnex_str, to_match):
	return bool(all(abnex_str, to_match))


def all(abnex_str, to_match):
	regex = parse(abnex_str)
	return re.findall(regex, to_match)


def split(abnex_str, to_match, max_split=0):
	regex = parse(abnex_str)
	if max_split:
		return re.split(regex, to_match, max_split)
	return re.split(regex, to_match)


def replace(abnex_str, to_match, replacement, count=0):
	regex = parse(abnex_str)
	return re.sub(regex, replacement, to_match, count)


def replace_count(abnex_str, to_match, replacement):
	regex = parse(abnex_str)
	return re.subn(regex, replacement, to_match)


def first(abnex_str, to_match):
	regex = parse(abnex_str)
	return re.search(regex, to_match)


def last(abnex_str, to_match):
	regex = parse(abnex_str)
	return re.findall(regex, to_match)[-1]


def escape(txt):
	escaped = ''

	for char in txt:
		prefix = ''
		if char in ['.', '$', '*', '+', '?', '(', ')', '[', '{', '\\', '-']:
			prefix = '\\'
		escaped += prefix + char

	return escaped


def get_chars():
	return {
		'(': '{',
		')': '}',
		'{': '(',
		'}': ')',
		'd': '\\d',
		'*': '.',
		'w': '\\w',
		'_': '\\s',
		':': '\\b',
		'->': '^',
		'<-': '$',
		's>': '\\A',
		'<s': '\\Z',
		'w>': '\\<',
		'<w': '\\>',
		'!': '^',
		'0++': '*',
		'1++': '+',
		'0+': '?',
		'c': '\\c',
		'x': '\\x',
		'o': '\\o',
	}


def transpile(char, is_not):
	chars = get_chars()
	translation = defaultdict(lambda: char, chars)[char]
	return translation if not is_not else translation.upper()


def parse(expr):
	regex = ''
	tmp = ''

	is_exact = False
	is_skip = False
	is_not = False
	is_quantifier = False
	tmp_range = ''
	tmp_range_ends_next = False

	for index, char in enumerate(expr):
		if tmp_range_ends_next:
			tmp_range += char
			regex += tmp_range

			tmp_range_ends_next = False
			tmp_range = ''
			continue

		if tmp_range and char in ['-']:
			tmp_range += char
			tmp_range_ends_next = True
		elif is_quantifier:
			tmp += char
			try:
				check = expr[index + 1] != '+'
			except IndexError:
				check = True

			if check:
				regex += transpile(tmp, is_not)
				tmp = ''
				is_quantifier = False
		elif is_skip:
			is_skip = False
		elif is_exact:
			if char in ['"', "'"]:
				is_exact = False
				tmp = escape(tmp)
				regex += tmp
				tmp = ''
			else:
				tmp += char
		elif char not in [' ', '\n', '\t']:
			if char in ['"', "'"]:
				is_exact = True
			elif char == '!' and expr[index - 1] != '[':
				is_not = True
			elif char in ['0', '1'] and expr[index + 1] == '+':
				is_quantifier = True
				tmp = char
			elif char in ['a', '0']:
				tmp_range = char
			else:
				if index < len(expr) - 1:
					next_up = expr[index + 1]
					if char in ['-', 's', 'w'] and next_up == '>' or char == '<' and next_up in ['-', 's', 'w']:
						is_skip = True
						regex += transpile(char + next_up, is_not)
					else:
						regex += transpile(char, is_not)
				else:
					regex += transpile(char, is_not)

	return regex



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['abnex']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'abnex',
    'version': '1.0.0',
    'description': 'Abnormal expressions (abnex) is an alternative to regular expressions (regex).',
    'long_description': '# Abnormal expressions\nAbnormal expressions (abnex) is an alternative to regular expressions (regex).\nThis is a Python library but the abnex syntax could be ported to other languages.\n\n# Examples\n## Matching an email address\n- Regex\n  - `([\\w\\._-]+)@([\\w\\.]+)`\n- Abnex\n  - `{[w"._-"]1++}"@"{[w"."]1++}`\n- Abnex (spaced)\n  - `{[w "._-"]1++} "@" {[w "."]1++}`\n\n- Abnex (expanded)\n  ```\n  {\n    [w "._-"]1++\n  }\n  "@"\n  {\n    [w "."]1++\n  }\n  ```\n  ## A more advanced pattern:\n  `{{{[a-z \'_\']1++} {[a-z 0-9 \'_-.\']0++}} \'@\' {{[a-z 0-9]1++} \'.\' {[a-z 0-9]1++} {[a-z 0-9 \'-_.\']0++}} {[a-z 0-9]1++}}`.\n\n# Why is Abnex Better?\n- It\'s easier to read, write and understand.\n- You can use spaces inside of the expression, you can also "expand" it, i.e. write it over multiple lines and use indention.\n- You don\'t have to use a backslashes all the time\n- More logical/common symbols like `!` for _not_, `{}` for _groups_, `1++`, `0++`, `0+` for: _one or more_, _zero or more_, _zero or one_.\n- It\'s easier to see if a symbol is an actual symbol you are searching for or if it\'s a regex character, ex:\n  - Regex: `[\\w-]+@[\\w-_]+`\n  - Abnex: `[w "-"]1++ "@" [w "-"]1++`\n\n# Documentation\n### Regex on right after -> is the abnex equivalent\n## Anchors\n- Start of string, or start of line in multi-line pattern\n  - `^` -> `->`\n- End of string, or end of line in multi-line pattern\n  - `$` -> `<-`\n- Start of string\n  - `\\A` -> `s>`\n- End of string\n  - `\\Z` -> `<s`\n- Word boundary\n  - `\\b` -> `:`\n- Not word boundary\n  - `\\B` -> `!:`\n- Start of word\n  - `\\<` -> `w>`\n- End of word\n  - `\\>` -> `<w`\n\n## Character Classes\n- Control character\n  - `\\c` -> `c`\n- White space\n  - `\\s` -> `_`\n- Not white space\n  - `\\S` -> `!_`\n- Digit\n  - `\\d` -> `d`\n- Not digit\n  - `\\D` -> `!d`\n- Word\n  - `\\w` -> `w`\n- Not word\n  - `\\W` -> `!w`\n- Hexade\xadcimal digit\n  - `\\x` -> `x`\n- Octal digit\n  - `\\o` -> `o`\n\n## Quantifiers\n- 0 or more\n  - `*` -> `0++`\n- 1 or more\n  - `+` -> `1++`\n- 0 or 1\n  - `?` -> `0+`\n\n## Groups and Ranges\n- Any character except new line (\\n)\n  - `.` -> `*`\n- a or b\n  - `a|b` -> `"a"|"b"`\n- Group\n  - `(...)` -> `{...}`\n- Passive (non-c\xadapt\xaduring) group\n  - `(?:...)` -> `{#...}`\n- Range (a or b or c)\n  - `[abc]` -> `[\'abc\']` or `["a" "b" "c"]`\n- Not in set\n  - `[^...]` -> `[!...]`\n- Lower case letter from a to Z\n  - `[a-q]` -> `[a-z]`\n- Upper case letter from A to Q\n  - `[A-Q]` -> `[A-Q]`\n- Digit from 0 to 7\n  - `[0-7]` -> `[0-7]`\n\n# Standards\nWhat is the recommended way to write abnexes\n\n- Use spaces between characters in character sets:\n  - Correct: `[w "_-"]`\n  - Incorrect: `[w"_-"]`\n- Put multiple exact characters between the same quotes in character sets:\n  - Correct: `["abc"]`\n  - Incorrect: `["a" "b" "c"]`, especially incorrect: `["a""b""c"]`\n- Put spaces between groups:\n  - Correct: `{w} "." {w}`\n  - Incorrect: `{w}"."{w}`\n\n### Examples:\nMatch for an email address:\n- Regex:\n  - `[\\w-\\._]+@[\\w-\\.]+`\n- Abnex (following standards):\n  - `{[w "-._"]1++} "@" {[w "-."]1++}`\n- Abnex (not following standards):\n  - `{[w"-._"]1++}"@"{[w"-."]1++}`\n  \n# Functions (In Python)\n__Abnex has most functions from the `re` library, but it also has som extra functionality like: `last()` & `contains()`.__\n\n## Common functions between re and abnex\n### Regex on right after -> is the abnex equivalent\n- `match()` -> `match()`\n- `findall()` -> `all()`\n- `split()` -> `split()`\n- `sub()` -> `replace()`\n- `subn()` -> `replace_count()`\n- `search()` -> `first()`\n## Special to abnex\n- `holds()`: whether or not a string matches an expression (bool).\n- `contains()`: wheter or not a string contains a match (bool).\n- `last()`: the last match in a string.\n',
    'author': 'Edvard Busck-Nielsen',
    'author_email': 'me@edvard.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Buscedv/abnormal-expressions',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
